/*
 * Copyright (C) 2023 Cobblemon Contributors
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */

"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var abilities_exports = {};
__export(abilities_exports, {
  Abilities: () => Abilities
});
module.exports = __toCommonJS(abilities_exports);
const Abilities = {
  damp: {
    inherit: true,
    onAnyDamage(damage, target, source, effect) {
      if (effect && (effect.id === "aftermath" || effect.id === "ability:aftermath")) {
        return false;
      }
    }
  },
  flowerveil: {
    inherit: true,
    onAllySetStatus(status, target, source, effect) {
      if (target.hasType("Grass") && source && target !== source && effect && effect.id !== "yawn") {
        this.debug("interrupting setStatus with Flower Veil");
        if (effect.id.endsWith("synchronize") || effect.effectType === "Move" && !effect.secondaries) {
          const effectHolder = this.effectState.target;
          this.add("-block", target, "ability: Flower Veil", "[of] " + effectHolder);
        }
        return null;
      }
    }
  },
  innerfocus: {
    inherit: true,
    onBoost(boost, target, source, effect) {
      if (effect.id === "intimidate" || effect.id === "ability:intimidate") {
        delete boost.atk;
        this.add("-fail", target, "unboost", "Attack", "[from] ability: Inner Focus", "[of] " + target);
      }
    }
  },
  mirrorarmor: {
    inherit: true,
    onBoost(boost, target, source, effect) {
      if (target === source || !boost || effect.id === "mirrorarmor" || effect.id === "ability:mirrorarmor")
        return;
      let b;
      for (b in boost) {
        if (boost[b] < 0) {
          if (target.boosts[b] === -6)
            continue;
          const negativeBoost = {};
          negativeBoost[b] = boost[b];
          delete boost[b];
          this.add("-ability", target, "Mirror Armor");
          this.boost(negativeBoost, source, target, null, true);
        }
      }
    }
  },
  neutralizinggas: {
    inherit: true,
    // Ability suppression implemented in sim/pokemon.ts:Pokemon#ignoringAbility
    onPreStart(pokemon) {
      this.add("-ability", pokemon, "Neutralizing Gas");
      pokemon.abilityState.ending = false;
      for (const target of this.getAllActive()) {
        if (target.hasItem("Ability Shield")) {
          this.add("-block", target, "item: Ability Shield");
          continue;
        }
        if (target.illusion) {
          this.singleEvent("End", this.dex.abilities.get("Illusion"), target.abilityState, target, pokemon, "neutralizinggas");
        }
        if (target.volatiles["slowstart"]) {
          delete target.volatiles["slowstart"];
          this.add("-end", target, "Slow Start", "[silent]");
        }
        if (target.m.innate) {
          if (!this.dex.abilities.get(target.m.innate.slice(8)).isPermanent) {
            target.removeVolatile(target.m.innate);
          }
        }
      }
    },
    onEnd(source) {
      this.add("-end", source, "ability: Neutralizing Gas");
      if (source.abilityState.ending)
        return;
      source.abilityState.ending = true;
      const sortedActive = this.getAllActive();
      this.speedSort(sortedActive);
      for (const pokemon of sortedActive) {
        if (pokemon !== source) {
          this.singleEvent("Start", pokemon.getAbility(), pokemon.abilityState, pokemon);
        }
        if (pokemon.m.innate) {
          if (!pokemon.volatiles[pokemon.m.innate])
            pokemon.addVolatile(pokemon.m.innate, pokemon);
        }
      }
    }
  },
  oblivious: {
    inherit: true,
    onBoost(boost, target, source, effect) {
      if (effect.id === "intimidate" || effect.id === "ability:intimidate") {
        delete boost.atk;
        this.add("-fail", target, "unboost", "Attack", "[from] ability: Oblivious", "[of] " + target);
      }
    }
  },
  owntempo: {
    inherit: true,
    onBoost(boost, target, source, effect) {
      if (effect.id === "intimidate" || effect.id === "ability:intimidate") {
        delete boost.atk;
        this.add("-fail", target, "unboost", "Attack", "[from] ability: Own Tempo", "[of] " + target);
      }
    }
  },
  rattled: {
    inherit: true,
    onAfterBoost(boost, target, source, effect) {
      if (effect && (effect.id === "intimidate" || effect.id === "ability:intimidate")) {
        this.boost({ spe: 1 });
      }
    }
  },
  scrappy: {
    inherit: true,
    onBoost(boost, target, source, effect) {
      if (effect.id === "intimidate" || effect.id === "ability:intimidate") {
        delete boost.atk;
        this.add("-fail", target, "unboost", "Attack", "[from] ability: Scrappy", "[of] " + target);
      }
    }
  },
  trace: {
    inherit: true,
    onUpdate(pokemon) {
      if (!pokemon.isStarted || this.effectState.gaveUp)
        return;
      const isAbility = pokemon.ability === "trace";
      const additionalBannedAbilities = [
        // Zen Mode included here for compatability with Gen 5-6
        "noability",
        "flowergift",
        "forecast",
        "hungerswitch",
        "illusion",
        "imposter",
        "neutralizinggas",
        "powerofalchemy",
        "receiver",
        "trace",
        "zenmode"
      ];
      const possibleTargets = pokemon.adjacentFoes().filter((target2) => !target2.getAbility().isPermanent && !additionalBannedAbilities.includes(target2.ability));
      if (!possibleTargets.length)
        return;
      const target = this.sample(possibleTargets);
      const ability = target.getAbility();
      this.add("-ability", pokemon, ability, "[from] ability: Trace", "[of] " + target);
      if (isAbility) {
        pokemon.setAbility(ability);
      } else {
        pokemon.removeVolatile("ability:trace");
        pokemon.addVolatile("ability:" + ability.id, pokemon);
      }
    }
  }
};
//# sourceMappingURL=abilities.js.map
